"
 COPYRIGHT (c) 2006 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:libbasic' }"

Object subclass:#StandaloneStartup
	instanceVariableNames:''
	classVariableNames:'Verbose CommandLineArguments'
	poolDictionaries:''
	category:'System-Support'
!

StandaloneStartup class instanceVariableNames:'MutexHandle'

"
 No other class instance variables are inherited by this class.
"
!

!StandaloneStartup class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2006 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    a subclassable template class for a standalone GUI-application's startup;
    For your own stand alone programs, define a subclass of this, 
    and redefine the #main: method there.
    (of course, the other methods can also be redefined.)

    [author:]
        Claus Gittinger

    [start with:]
        <yourNamehere>Startup start

    [see also:]
        Smalltalk
        GetOpt
        ReadEvalPrintLoop
"
!

howToDealWithMultipleApplicationInstances
"
    please read the comment in the corresponding ApplicationModel class-documentation method.
"
!

whichMethodsToRedefine
"
    main:argv
        thats the actual program.

    suppressRCFileReading
        false here; redefine to return true, to disable the rc-file reading.
        you loose the chance of configuration, but lock the user out from any access to any smalltalk
        (if you have a user-phobia)

    allowDebugOption
        false here; redefine to return true, to enable the --debug startup option.
        if disabled, you loose the chance of debugging, but lock the user out from any access to any smalltalk

    allowScriptingOption
        false here; redefine to return true, to enable the --scripting startup option.
        if disabled, you loose the chance of remote control, but lock the user out from any access to any smalltalk

"
! !

!StandaloneStartup class methodsFor:'initialization'!

initialize
    "/ Verbose := true.
    Verbose := false.
! !

!StandaloneStartup class methodsFor:'defaults'!

allowDebugOption
    "enable/disable the --debug startup option.
     The default is now false, so standAlone apps are closed by default.
     Can be redefined in subclasses to enable it"

    ^ false
!

allowScriptingOption
    "enable/disable the --scripting startup option.
     The default is now false, so standAlone apps are closed by default.
     Can be redefined in subclasses to enable it"

    ^ false
!

suppressRCFileReading
    "enable/disable the rc-file reading (and also the --rcFileName option).
     If suppressed, there is no chance to interfere with the startup.
     Can be redefined in subclasses to disable it"

    ^ false
! !

!StandaloneStartup class methodsFor:'helpers'!

printInfo:msg
    Transcript 
        show:('%1 [info]: ' bindWith:(self applicationName));
        showCR:msg
!

redirectStandardStreams
    Stdout := Stderr.
    Transcript := Stderr.
!

verboseInfo:msg
    Verbose == true ifFalse:[^ self].
    self printInfo:msg

    "Modified: / 19-09-2006 / 16:30:27 / cg"
! !

!StandaloneStartup class methodsFor:'multiple applications support'!

applicationRegistryPath
    "the key under which this application stores its process ID in the registry
     as a collection of path-components.
     i.e. if #('foo' 'bar' 'baz') is returned here, the current applications ID will be stored
     in HKEY_CURRENT_USER\Software\foo\bar\baz\CurrentID.
     (would also be used as a relative path for a temporary lock file under unix).
     Used to detect if another instance of this application is already running."

    self subclassResponsibility
!

applicationUUID
    "answer an application-specific unique uuid.
     This is used as the name of some exclusive OS-resource, which is used to find out,
     if another instance of this application is already running.
     Under win32, a mutex is used; under unix, an exclusive file in the tempDir could be used."
    
    self subclassResponsibility
!

shouldReuseRunningApplication
    "answer true, if an already running application instance should be re-used"

    ^ false
! !

!StandaloneStartup class methodsFor:'multiple applications support-helpers'!

applicationRegistryEntry
    "retrieve the registry entry in which (if present), any currently running application
     has left its process ID"

    |path relPathName applicationEntry softwareEntry|

    path := self applicationRegistryPath.
    relPathName := path asStringWith:$\.
    applicationEntry := Win32OperatingSystem registryEntry key:'HKEY_CURRENT_USER\Software\',relPathName.
    applicationEntry isNil ifTrue:[
        softwareEntry := Win32OperatingSystem registryEntry key:'HKEY_CURRENT_USER\Software'.
        softwareEntry isNil ifTrue:[
            Transcript showCR: 'Failed to get Software entry in registry'.
            ^ nil.
        ].

        path do:[:subKey |
            |subEntry|

            subEntry := softwareEntry createSubKeyNamed:subKey.
            subEntry isNil ifTrue:[
                Transcript showCR: 'Failed to create ',subKey,' entry in registry'.
                ^ nil.
            ].
            softwareEntry := subEntry.
        ].
        applicationEntry := softwareEntry.
    ].
    ^ applicationEntry
!

checkForAndExitIfAnotherApplicationInstanceIsRunning
    "if another instance of this application is running,
     send it an openFile command for my file-argument, and exit.
     (i.e. to let the already running application open up another window)."

    |shouldExit|

    self isAnotherApplicationInstanceRunning ifTrue:[
        shouldExit := self processStartupOfASecondInstance.
        shouldExit ifTrue:[
            self releaseApplicationMutex.
            Smalltalk isStandAloneApp ifTrue:[
                Smalltalk exit.
            ]
        ].
    ].

    "Modified: / 03-08-2010 / 17:27:25 / cg"
!

confirmOpenNewApplicationInstance

    ^ Dialog confirm: ('Continue opening a new instance of %1 or exit?' bindWith:self applicationName)
                title: ('%1 is already open!!' bindWith:self applicationName)
             yesLabel: 'Continue' 
              noLabel: 'Exit'
!

currentIDKeyInRegistry
    ^ 'CurrentID'
!

getCurrentIDFromRegistry

    |applicationEntry|

    applicationEntry := self applicationRegistryEntry.
    applicationEntry isNil ifTrue:[^ nil.].
    ^ applicationEntry valueNamed: self currentIDKeyInRegistry

    "
     |hWnd externalAddress|
     hWnd := DapasXStartup getCurrentIDFromRegistry.   
     hWnd isEmptyOrNil ifTrue:[^ self halt.].
     hWnd := hWnd asInteger.
     externalAddress := ExternalAddress newAddress: hWnd.
     Display raiseWindow:externalAddress.
     Display setForegroundWindow:externalAddress
    "
!

getIDOfRunningApplicationFromRegistryEntry
    |applicationEntry|

    applicationEntry := self applicationRegistryEntry.
    applicationEntry isNil ifTrue:[^ nil.].
    ^ applicationEntry valueNamed: self currentIDKeyInRegistry
!

isAnotherApplicationInstanceRunning
    "answer true, if another instance of myself is currently running.
     For now, it only works under win32, because it uses the underlying mutex mechanism."
    
    | lastErrorCode alreadyExists handleAndLastErrorCode |

    OperatingSystem isMSDOSlike ifTrue:[
        handleAndLastErrorCode := OperatingSystem createMutexNamed: (self applicationUUID printString).
        MutexHandle := handleAndLastErrorCode first.
        lastErrorCode := handleAndLastErrorCode second.
        "/ self assert: lastErrorCode == 0.
        alreadyExists := 
            MutexHandle isNil 
            or:[lastErrorCode == 183 "ERROR_ALREADY_EXISTS"
            or:[lastErrorCode == 5 "ERROR_ACCESS_DENIED"]].

        alreadyExists ifFalse:[OperatingSystem waitForSingleObject: MutexHandle].
        ^ alreadyExists
    ].

    ^ false.

    "Modified: / 01-10-2010 / 18:27:29 / cg"
!

processStartupOfASecondInstance
    "This is executed when I have been started as a second instance of an already running application.
     If I can get the currentID (i.e. windowID) of the first one and there is a command line argument with a file, 
     send a message to the main window of the already running application, to ask it for another window.
     If the currentID is unknown, ask if the user wants to open a new instance of the application anyway.
     Return true if the first instance has been notified, and this second instance should exit."

    |currentIDStringFromRegistry currentIDFromRegistry fileArg commands aWindowId setForegroundWindowSucceeded|

    commands := CommandLineArguments.

    currentIDStringFromRegistry := self getCurrentIDFromRegistry.

    "If the currentID is not found and there are arguments from the command line, 
     we should wait in case of starting the first instance of the application 
     with a multiple selection of files."
    (currentIDStringFromRegistry isEmptyOrNil and:[commands notEmptyOrNil]) ifTrue:[
        Delay waitForSeconds: 2.
    ].

    currentIDStringFromRegistry := self getCurrentIDFromRegistry.
    currentIDStringFromRegistry isEmptyOrNil ifTrue:[
        ^ self confirmOpenNewApplicationInstance not.
    ].

    currentIDFromRegistry := Integer readFrom:currentIDStringFromRegistry onError: 0.

    "/ bring the other application to the foreground
    aWindowId := ExternalAddress newAddress: currentIDFromRegistry.
    setForegroundWindowSucceeded := Display primSetForegroundWindow: aWindowId.
"/    setForegroundWindowSucceeded ifFalse:[^ self confirmOpenNewApplicationInstance not].

    "Autostart for associated extension"
    commands notEmpty ifTrue:[
        fileArg := commands last asFilename.
        fileArg exists ifTrue:[
            self sendOpenPathCommand:(fileArg pathName) toWindowId: aWindowId.
        ].
    ].
    ^ true

    "Modified: / 08-07-2010 / 00:47:44 / cg"
!

releaseApplicationMutex

    (MutexHandle notNil and:[OperatingSystem isMSDOSlike]) ifTrue:[
        OperatingSystem releaseMutex: MutexHandle.
        OperatingSystem primCloseHandle: MutexHandle.

        MutexHandle := nil.
    ].
!

sendCommand:message toWindowId:aWindowId
    "use the event send mechanism to forward a command to the already running application"

    Display 
        sendCopyDataString: message 
        toWindowId: aWindowId.
!

sendOpenPathCommand:pathName toWindowId:aWindowId
    "use the event send mechanism to forward an open-Path command to the already running application"

    self sendCommand:('openPath:', pathName) toWindowId:aWindowId.
!

writeCurrentIDIntoRegistry: currentID

    |applicationEntry currentIDEntry|

    applicationEntry := self applicationRegistryEntry.
    applicationEntry isNil ifTrue:[^ false.].

    currentIDEntry := applicationEntry createSubKeyNamed:(self currentIDKeyInRegistry).
    currentIDEntry isNil ifTrue:[
        Transcript showCR: 'Failed to create CurrentID entry in registry'.
        ^ false.
    ].

    ^ applicationEntry valueNamed:(self currentIDKeyInRegistry) put:(currentID printString).

    "
     | currentID returnedCurrentID |
     currentID := 999.
     DapasXStartup writeCurrentIDIntoRegistry: currentID.
     returnedCurrentID := DapasXStartup getCurrentIDFromRegistry.
     self assert: currentID = returnedCurrentID asNumber.
    "
! !

!StandaloneStartup class methodsFor:'queries'!

applicationName
    "used in verbose messages - can/should be redefined in subclasses"

    |nm|

    nm := self nameWithoutPrefix.
    (nm endsWith:'Startup') ifTrue:[
        ^ nm copyWithoutLast:('Startup' size).
    ].
    (nm endsWith:'Start') ifTrue:[
        ^ nm copyWithoutLast:('Start' size).
    ].
    ^ nm

    "Created: / 19-09-2006 / 16:26:44 / cg"
!

iconInBrowserSymbol
    <resource: #programImage>

    self == StandaloneStartup ifFalse:[
        ^ #standaloneStartupIcon 
    ].
    ^ super iconInBrowserSymbol
!

isBrowserStartable
    "do not allow clicking on me in the browser"

    ^ false

    "Created: / 06-10-2006 / 11:33:13 / cg"
!

keepSplashWindowOpen
    "if true is returned here, the splashWindow is not closed and will be still open
     when the main: method is invoked. This allows for plugin-loads etc. to be shown in the
     splash screen. However, my subclass's main: has to make sure that the splashScreen is closed.
     (calling hideSplashWindow)
     The default is false here which means that the splashWindow will be already closed when the
     subclasses main: is invoked."

    ^ false
!

startupFilename
    "used in verbose messages - can/should be redefined in subclasses"

    ^ self applicationName asLowercase,'Start.rc'

    "
     ExpeccoStartup startupFilename -> 'expecco.rc'
    "

    "Created: / 19-09-2006 / 16:38:28 / cg"
! !

!StandaloneStartup class methodsFor:'startup'!

loadPatch:fileName
    self verboseInfo:('loading patch: ',fileName baseName).
    Smalltalk fileIn:fileName pathName.
!

loadPatches
    |patchesDir prevMode|

    patchesDir := OperatingSystem pathOfSTXExecutable asFilename directory construct:'patches'.
    (patchesDir exists and:[patchesDir isDirectory]) ifTrue:[
        prevMode := ClassCategoryReader sourceMode.
        ClassCategoryReader sourceMode:#discard.
        [
            patchesDir directoryContents sort do:[:eachFilenameString |
                self loadPatch:(patchesDir construct:eachFilenameString).
            ].
        ] ensure:[
            ClassCategoryReader sourceMode:prevMode.
        ].
    ].

    "Modified: / 19-09-2006 / 16:30:58 / cg"
!

loadRemainingClassLibraries
    "To speedup startup, we did not load all dll's (only a subset of non-GUI dll's is present).
     Now, load all skipped libs (the ones marked with '*') from modules.stx."

    |modulesFile dllDirectory dlls|

    OperatingSystem isMSWINDOWSlike ifFalse:[^ self ].

    self verboseInfo:'loadRemainingClassLibraries'.
    modulesFile  := self stxModulesFilename.
    dllDirectory := modulesFile directory.

    dlls := OrderedCollection new.

    modulesFile readingLinesDo:[:eachModulesLine|
        |basename dllFile|

        basename := eachModulesLine withoutSeparators.

        (basename notEmpty and:[basename first == $*]) ifTrue:[
            basename := (basename copyFrom:2) withoutSeparators, '.dll'.
            dllFile := dllDirectory construct:basename.

            dllFile exists ifTrue:[
"/                self verboseInfo:('loading: ', basename).
                Smalltalk showSplashMessage:('loading ', basename).
                dlls add:dllFile.
            ] ifFalse:[
                self verboseInfo:( '**** cannot resolve: ', basename).
            ].
        ].
    ].

    dlls notEmpty ifTrue:[
        ObjectFileLoader loadObjectFiles:dlls.
        Display notNil ifTrue:[
            "New view classes may have been loaded - have to update their styles"
            self verboseInfo:'update style caches of loaded dlls'.
            SimpleView readStyleSheetAndUpdateAllStyleCaches.
        ].
    ].
!

setupSmalltalkFromArguments:argv
    "handle common command line arguments:
        --help ............... print usage and exit
        --verbose (-V) ....... be verbose during startup
        --debug .............. enable debugger & inspector
        --rcFileName ......... define a startup rc-file
        --scripting portNr ... start a scripting server
        --allowHost host ..... add host to the allowed scripting hosts
    "

    |idx rcFilename nextArg debugging scripting allowedScriptingHosts portNr|

"/    Smalltalk beHeadless:true.
"/    OperatingSystem disableSignal:(OperatingSystem sigHUP).
"/    Smalltalk infoPrinting:true.

    (argv includes:'--help') ifTrue:[
        self usage.
        AbortOperationRequest raise.
    ].

    idx := argv indexOfAny:#('--verbose' '-V').
    idx ~~ 0 ifTrue:[
        argv removeAtIndex:idx.
        Verbose := true.
    ].
    self verboseInfo:('args: ',argv asArray printString).

    debugging := false.
    (self allowDebugOption) ifTrue:[
        idx := argv indexOfAny:#('--debug').
        idx ~~ 0 ifTrue:[
            self verboseInfo:('debug on').
            argv removeAtIndex:idx.
            debugging := true
        ].
    ].
    debugging ifTrue:[
        self setupToolsForDebug.
    ] ifFalse:[
        self setupToolsForNoDebug.
    ].

    self suppressRCFileReading ifFalse:[
        idx := argv indexOf:'--rcFileName'.
        idx ~~ 0 ifTrue:[
            nextArg := argv at:(idx + 1) ifAbsent:nil.
            (nextArg notNil and:[ (nextArg startsWith:'-') not ]) ifTrue:[
                rcFilename := nextArg.
                argv removeAtIndex:idx+1; removeAtIndex:idx.
            ]
        ].

        rcFilename isNil ifTrue:[
            rcFilename := self startupFilename.
        ].
        rcFilename asFilename exists ifTrue:[
            self verboseInfo:('reading ',rcFilename,'...').
            rcFilename isAbsolute ifFalse:[
                rcFilename := OperatingSystem pathOfSTXExecutable asFilename directory constructString:rcFilename.
            ].
            Smalltalk secureFileIn:rcFilename
        ].
    ].

    scripting := false.
    (self allowScriptingOption) ifTrue:[
        idx := argv indexOfAny:#('--scripting').
        idx ~~ 0 ifTrue:[
            nextArg := argv at:(idx + 1) ifAbsent:nil.
            (nextArg notNil and:[ (nextArg startsWith:'-') not ]) ifTrue:[
                portNr := nextArg asInteger.
                argv removeAtIndex:idx+1.
            ].
            argv removeAtIndex:idx.

            scripting := true
        ].

        allowedScriptingHosts := OrderedCollection new.

        idx := argv indexOfAny:#('--allowHost').
        [idx ~~ 0] whileTrue:[
            nextArg := argv at:(idx + 1) ifAbsent:nil.
            nextArg isNil ifTrue:[
                self usage.
                AbortOperationRequest raise.
            ].
            allowedScriptingHosts add:nextArg.
            idx := argv indexOfAny:#('--allowHost').
        ].
    ].

    scripting ifTrue:[
        self verboseInfo:('scripting on').
        STXScriptingServer notNil ifTrue:[
            allowedScriptingHosts do:[:eachHost | STXScriptingServer allowHost:eachHost ].

            "/ scripting on port/stdin_out/8008
            self verboseInfo:('start scripting').
            STXScriptingServer startAt:portNr
        ] ifFalse:[
            self verboseInfo:('missing STXScriptingServer class').
        ].
    ].

    ^ true

    "Modified: / 31-10-2007 / 16:05:59 / cg"
!

setupToolsForDebug
    Debugger := DebugView ? MiniDebugger.
    Inspector := InspectorView ? MiniInspector.
    Verbose ifTrue:[ 'debug enabled - CTRL-C brings you into a debugger.' errorPrintCR ].
"/    self verboseInfo:('debug enabled - CTRL-C brings you into a debugger.').

    "Created: / 19-09-2006 / 16:40:32 / cg"
!

setupToolsForNoDebug
    Smalltalk isStandAloneApp ifTrue:[
        Smalltalk at:#Debugger put:nil.
        Smalltalk at:#Inspector put:nil.
        NewLauncher notNil ifTrue:[
            NewLauncher allPrivateClasses do:[:cls |
                Smalltalk at:(cls name) put:nil.
            ].
            Smalltalk at:#NewLauncher put:nil.
        ].
        Verbose ifTrue:[ 'debug disabled.' errorPrintCR ].

        self redirectStandardStreams.
    ].

    "Created: / 19-09-2006 / 16:40:47 / cg"
    "Modified: / 31-10-2007 / 16:18:40 / cg"
!

start
    GenericException handle:[:ex |
        self verboseInfo:('Error during startup:').
        self verboseInfo:(ex description).
        Verbose == true ifTrue:[ex suspendedContext fullPrintAllLevels:10].
        ex reject.        
    ] do:[
        |idx|
        self verboseInfo:('starting...').
        CommandLineArguments := (self additionalArgumentsFromRegistry) 
                                , Smalltalk commandLineArguments.

        self verboseInfo:('args: ', CommandLineArguments asStringCollection asString).

        "--noRemote - do not reuse an existing application instance,
         but run in a separate process"
        idx := CommandLineArguments indexOfAny:#('--noRemote').
        idx == 0 ifTrue:[
            self shouldReuseRunningApplication ifTrue:[
                "Multiple Application support:
                 if another expecco is running, ask it to open another window for me.
                 If that is the case, the following function will not return, but instead exit."
                self checkForAndExitIfAnotherApplicationInstanceIsRunning.
            ].
        ] ifFalse:[
            CommandLineArguments removeAtIndex:idx.
            Verbose := true.
        ].

        "/ Arrive here, if no other application is running.
        "/ to speedup startup, we did not load all dll's (only a subset of non-GUI dll's is present).
        "/ now, load all skipped libs from modules.stx.
        self loadRemainingClassLibraries.

        Smalltalk isStandAloneApp ifTrue:[
            self loadPatches.
            self verboseInfo:('setup Smalltalk').
        ].
        self setupSmalltalkFromArguments:CommandLineArguments.
        self main
    ].

    "Modified: / 04-08-2010 / 12:30:05 / cg"
!

startStartBlockProcess
    Smalltalk startStartBlockProcess
!

stxModulesFilename
    "answer the Filename of modules.stx"

    |file|

    file := 'modules.stx' asFilename.
    file exists ifTrue:[^ file].

    file := OperatingSystem pathOfSTXExecutable asFilename directory construct:'modules.stx'.
    file exists ifTrue:[^ file].

    self error:'cannot find: modules.stx'.
!

usage
    Stderr nextPutLine:'usage:'.
    Stderr nextPutLine:'  ',self applicationName,' [options...]'.
    Stderr nextPutLine:'    options:'.
    Stderr nextPutLine:'          --help .................. output this message'.
    Stderr nextPutLine:'          --verbose ............... verbose startup'.
    Stderr nextPutLine:'          --noBanner .............. no splash screen'.
    Stderr nextPutLine:'          --noRemote .............. start as its own application process (do not reuse a running instance)'.
    self allowScriptingOption ifTrue:[
        Stderr nextPutLine:'          --scripting portNr ...... enable scripting via port (or stdin/stdOut, if 0)'.
    ].
    self allowDebugOption ifTrue:[
        Stderr nextPutLine:'          --debug ................. enable Debugger'.
    ].
    self suppressRCFileReading ifFalse:[
        Stderr nextPutLine:'          --rcFileName file ....... execute code from file on startup (default: ',self startupFilename,')'.
    ].

    "Created: / 19-09-2006 / 16:37:55 / cg"
    "Modified: / 04-10-2010 / 10:47:48 / cg"
! !

!StandaloneStartup class methodsFor:'startup-to be redefined'!

additionalArgumentsFromRegistry
    "can be redefined to fetch and return additional arguments from the registry 
     (or other .ini file). These are added to the beginning of the command line arguments."

    ^ #()

    "Created: / 04-08-2010 / 12:20:27 / cg"
!

isHeadless
    "this is invoked early by Smalltalk>>mainStartup, to ask if I like to
     have a Display or if I am a non-GUI headless application.

     Redefine in subclass, if your application is a non-GUI application"

    ^ false
!

main
    self verboseInfo:('entering main').

    self main:CommandLineArguments.

    "
     self main
     self main:#('--info') 
    "

    "Modified: / 31-10-2007 / 16:03:22 / cg"
!

main:argv
    self subclassResponsibility.

"/ a typical main: looks like (in a subclass):

"/    |app fileArg|
"/
"/    self verboseInfo:('starting application').
"/    app := <someGUIApplicationModelClass> open.
"/
"/    self verboseInfo:('looking for args in ',argv).
"/    argv notEmptyOrNil ifTrue:[
"/        fileArg := argv last asFilename.
"/        self verboseInfo:('fileArg is ',fileArg name).
"/        fileArg exists ifTrue:[
"/            self verboseInfo:('file exists').
"/
"/            ( #('foo' 'bar' 'baz' ) includes:fileArg suffix) ifTrue:[
"/                self verboseInfo:('loading').
"/
"/                Error handle:[:ex |
"/                    self verboseInfo:'error while loading'.
"/                    ex suspendedContext fullPrintAll.
"/                ] do:[
"/                    app menuLoadFromFile:fileArg
"/                ].
"/            ].
"/        ].
"/    ].

    "Created: / 19-09-2006 / 16:48:29 / cg"
! !

!StandaloneStartup class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libbasic/StandaloneStartup.st,v 1.52 2010-10-04 11:54:02 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/libbasic/StandaloneStartup.st,v 1.52 2010-10-04 11:54:02 cg Exp $'
! !

StandaloneStartup initialize!
